@@ -87,7 +87,7 @@ enum ErrorContext<'e> {
     Stack(&'e StackContext),
 }
 
-impl<'e> Display for ErrorContext<'e> {
+impl Display for ErrorContext<'_> {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         match self {
             ErrorContext::Kind(kind) => match kind {
@@ -139,12 +139,12 @@ impl<'i> ErrorTreeExt<Input<'i>> for ErrorTree<'i> {
                         })
                     }
                     recurse(base, depth + 1, f);
-                }
+                },
                 ErrorTree::Alt(ref trees) => {
                     for tree in trees {
                         recurse(tree, depth + 1, f);
                     }
-                }
+                },
             }
         }
         recurse(self, 0, &mut f);
@@ -174,6 +174,17 @@ impl<'i> ErrorTreeExt<Input<'i>> for ErrorTree<'i> {
     }
 }
 
+/// Describes errors that occur when parsing a glob expression.
+///
+/// Common examples of glob expressions that cannot be parsed are alternative
+/// and repetition patterns with missing delimiters and ambiguous patterns, such
+/// as `src/***/*.rs` or `{.local,.config/**/*.toml`.
+///
+/// When the `diagnostics-report` feature is enabled, this error implements the
+/// [`Diagnostic`] trait and provides more detailed information about the parse
+/// failure.
+///
+/// [`Diagnostic`]: miette::Diagnostic
 #[derive(Clone, Debug, Error)]
 #[error("failed to parse glob expression")]
 pub struct ParseError<'t> {
@@ -187,7 +198,7 @@ impl<'t> ParseError<'t> {
         match error {
             ErrorMode::Incomplete(_) => {
                 panic!("unexpected parse error: incomplete input")
-            }
+            },
             ErrorMode::Error(error) | ErrorMode::Failure(error) => {
                 let (starts, ends) = error.bounding_error_locations();
                 ParseError {
@@ -198,10 +209,11 @@ impl<'t> ParseError<'t> {
                         .expect("expected lower bound error location"),
                     ends,
                 }
-            }
+            },
         }
     }
 
+    /// Clones any borrowed data into an owning instance.
     pub fn into_owned(self) -> ParseError<'static> {
         let ParseError {
             expression,
@@ -215,6 +227,7 @@ impl<'t> ParseError<'t> {
         }
     }
 
+    /// Gets the glob expression that failed to parse.
     pub fn expression(&self) -> &str {
         self.expression.as_ref()
     }
@@ -222,7 +235,7 @@ impl<'t> ParseError<'t> {
 
 #[cfg(feature = "diagnostics-report")]
 #[cfg_attr(docsrs, doc(cfg(feature = "diagnostics-report")))]
-impl<'t> Diagnostic for ParseError<'t> {
+impl Diagnostic for ParseError<'_> {
     fn code<'a>(&'a self) -> Option<Box<dyn Display + 'a>> {
         Some(Box::new("wax::glob::parse"))
     }
@@ -256,6 +269,12 @@ impl<'t> Diagnostic for ParseError<'t> {
     }
 }
 
+pub trait IntoTokens<'t>: Sized {
+    type Annotation;
+
+    fn into_tokens(self) -> Vec<Token<'t, Self::Annotation>>;
+}
+
 #[derive(Clone, Copy, Debug, Default)]
 struct ParserState {
     flags: FlagState,
@@ -295,7 +314,7 @@ impl<'t, A> Tokenized<'t, A> {
         }
     }
 
-    pub fn partition(&mut self) -> PathBuf {
+    pub fn partition(mut self) -> (PathBuf, Self) {
         // Get the invariant prefix for the token sequence.
         let prefix = invariant_prefix_path(self.tokens.iter()).unwrap_or_else(PathBuf::new);
 
@@ -305,7 +324,7 @@ impl<'t, A> Tokenized<'t, A> {
             .drain(0..invariant_prefix_upper_bound(&self.tokens));
         self.tokens.first_mut().map(Token::unroot);
 
-        prefix
+        (prefix, self)
     }
 
     pub fn expression(&self) -> &Cow<'t, str> {
@@ -317,6 +336,15 @@ impl<'t, A> Tokenized<'t, A> {
     }
 }
 
+impl<'t, A> IntoTokens<'t> for Tokenized<'t, A> {
+    type Annotation = A;
+
+    fn into_tokens(self) -> Vec<Token<'t, Self::Annotation>> {
+        let Tokenized { tokens, .. } = self;
+        tokens
+    }
+}
+
 #[derive(Clone, Debug)]
 pub struct Token<'t, A = Annotation> {
     kind: TokenKind<'t, A>,
@@ -336,6 +364,14 @@ impl<'t, A> Token<'t, A> {
         }
     }
 
+    pub fn unannotate(self) -> Token<'t, ()> {
+        let Token { kind, .. } = self;
+        Token {
+            kind: kind.unannotate(),
+            annotation: (),
+        }
+    }
+
     pub fn unroot(&mut self) -> bool {
         self.kind.unroot()
     }
@@ -443,11 +479,22 @@ impl<'t, A> TokenKind<'t, A> {
         }
     }
 
+    pub fn unannotate(self) -> TokenKind<'t, ()> {
+        match self {
+            TokenKind::Alternative(alternative) => TokenKind::Alternative(alternative.unannotate()),
+            TokenKind::Class(class) => TokenKind::Class(class),
+            TokenKind::Literal(literal) => TokenKind::Literal(literal),
+            TokenKind::Repetition(repetition) => TokenKind::Repetition(repetition.unannotate()),
+            TokenKind::Separator => TokenKind::Separator,
+            TokenKind::Wildcard(wildcard) => TokenKind::Wildcard(wildcard),
+        }
+    }
+
     pub fn unroot(&mut self) -> bool {
         match self {
             TokenKind::Wildcard(Wildcard::Tree { ref mut is_rooted }) => {
                 mem::replace(is_rooted, false)
-            }
+            },
             _ => false,
         }
     }
@@ -492,7 +539,7 @@ impl<'t, A> From<Alternative<'t, A>> for TokenKind<'t, A> {
     }
 }
 
-impl<'t, A> From<Class> for TokenKind<'t, A> {
+impl<A> From<Class> for TokenKind<'_, A> {
     fn from(class: Class) -> Self {
         TokenKind::Class(class)
     }
@@ -523,6 +570,16 @@ impl<'t, A> Alternative<'t, A> {
         )
     }
 
+    pub fn unannotate(self) -> Alternative<'t, ()> {
+        let Alternative(branches) = self;
+        Alternative(
+            branches
+                .into_iter()
+                .map(|branch| branch.into_iter().map(|token| token.unannotate()).collect())
+                .collect(),
+        )
+    }
+
     pub fn branches(&self) -> &Vec<Vec<Token<'t, A>>> {
         &self.0
     }
@@ -581,19 +638,19 @@ impl Archetype {
             Archetype::Character(x) => (!PATHS_ARE_CASE_INSENSITIVE).then(|| x.to_string().into()),
             Archetype::Range(a, b) => {
                 ((a == b) && !PATHS_ARE_CASE_INSENSITIVE).then(|| a.to_string().into())
-            }
+            },
         }
     }
 }
 
 impl From<char> for Archetype {
-    fn from(literal: char) -> Archetype {
+    fn from(literal: char) -> Self {
         Archetype::Character(literal)
     }
 }
 
 impl From<(char, char)> for Archetype {
-    fn from(range: (char, char)) -> Archetype {
+    fn from(range: (char, char)) -> Self {
         Archetype::Range(range.0, range.1)
     }
 }
@@ -703,6 +760,19 @@ impl<'t, A> Repetition<'t, A> {
         }
     }
 
+    pub fn unannotate(self) -> Repetition<'t, ()> {
+        let Repetition {
+            tokens,
+            lower,
+            step,
+        } = self;
+        Repetition {
+            tokens: tokens.into_iter().map(|token| token.unannotate()).collect(),
+            lower,
+            step,
+        }
+    }
+
     pub fn tokens(&self) -> &Vec<Token<'t, A>> {
         &self.tokens
     }
@@ -743,12 +813,6 @@ impl<'t, A> Repetition<'t, A> {
 pub enum Wildcard {
     One,
     ZeroOrMore(Evaluation),
-    // NOTE: Repetitions can represent tree wildcards as they are more general
-    //       and are allowed to cross component boundaries. However,
-    //       transforming tree wildcards into repetitions would move complex
-    //       logic into the parser and require additional state that is easier
-    //       (and likely less expensive) to manage in compilation (see
-    //       `encode::compile`).
     Tree { is_rooted: bool },
 }
 
@@ -824,6 +888,23 @@ impl<'i, 't, A> Clone for Component<'i, 't, A> {
     }
 }
 
+pub fn any<'t, A, I>(tokens: I) -> Token<'t, ()>
+where
+    I: IntoIterator,
+    I::Item: IntoIterator<Item = Token<'t, A>>,
+{
+    Token {
+        kind: Alternative(
+            tokens
+                .into_iter()
+                .map(|tokens| tokens.into_iter().map(|token| token.unannotate()).collect())
+                .collect(),
+        )
+        .into(),
+        annotation: (),
+    }
+}
+
 pub fn components<'i, 't, A, I>(tokens: I) -> impl Iterator<Item = Component<'i, 't, A>>
 where
     't: 'i,
@@ -871,13 +952,12 @@ where
                         alternative
                             .branches()
                             .iter()
-                            .map(literals)
-                            .flatten()
+                            .flat_map(literals)
                             .collect::<Vec<_>>(),
                     ),
                     TokenKind::Repetition(ref repetition) => {
                         Some(literals(repetition.tokens()).collect::<Vec<_>>())
-                    }
+                    },
                     _ => None,
                 })
                 .flatten()
@@ -930,10 +1010,10 @@ pub fn invariant_prefix_upper_bound<A>(tokens: &[Token<A>]) -> usize {
         match token {
             Separator => {
                 separator = Some(n);
-            }
+            },
             Wildcard(Tree { .. }) => {
                 return n;
-            }
+            },
             _ => {
                 // TODO: This may be expensive.
                 if token.to_invariant_string().is_some() {
@@ -945,7 +1025,7 @@ pub fn invariant_prefix_upper_bound<A>(tokens: &[Token<A>]) -> usize {
                         None => 0,
                     };
                 }
-            }
+            },
         }
     }
     tokens.len()
@@ -1012,7 +1092,7 @@ pub fn parse(expression: &str) -> Result<Tokenized, ParseError> {
                 match toggle {
                     CaseInsensitive(toggle) => {
                         input.state.flags.is_case_insensitive = toggle;
-                    }
+                    },
                 }
                 Ok((input, ()))
             }
